@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -20,22 +20,25 @@
 
 import copy
 import logging
-import datetime
 import time
+import datetime
+from math import ceil
 
-from sqlalchemy import Column, Integer, ForeignKey
-from sqlalchemy.orm import validates, relationship
+from sqlalchemy import (Column, Integer, ForeignKey, Interval, Boolean,
+                        DateTime, PickleType)
+from sqlalchemy.orm import validates, relationship, synonym, reconstructor
 
 from stalker import defaults, log
+from stalker.db.session import DBSession
 from stalker.models.entity import SimpleEntity, Entity
-from stalker.models.mixins import ScheduleMixin, WorkingHoursMixin
+from stalker.models.mixins import DateRangeMixin, WorkingHoursMixin
 from stalker.models.schedulers import SchedulerBase
 
 logger = logging.getLogger(__name__)
 logger.setLevel(log.logging_level)
 
 
-class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
+class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
     """Manage all the studio information at once.
 
     With Stalker you can manage all you Studio data by using this class. Studio
@@ -67,7 +70,21 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
 
     **Vacations**
 
-    You can define
+    Studio wide vacations are managed by the Studio class.
+
+    **Scheduling**
+
+    .. versionadded: 0.2.5
+       Schedule Info Attributes
+
+    There are a couple of attributes those become pretty interesting when used
+    together with the Studio instance while using the scheduling part of the
+    Studio. Please refer to the attribute documentation for each attribute:
+
+      :attr:`.is_scheduling`
+      :attr:`.last_scheduled_at`
+      :attr:`.last_scheduled_by`
+      :attr:`.last_schedule_message`
 
     :param int daily_working_hours: An integer specifying the daily working
       hours for the studio. It is another critical value attribute which
@@ -80,6 +97,15 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
       datetime.datetime.now().
 
     :type now: datetime.datetime
+
+    :param timing_resolution: The timing_resolution of the datetime.datetime
+      object in datetime.timedelta. Uses ``timing_resolution`` settings in the
+      :class:`stalker.config.Config` class which defaults to 1 hour. Setting
+      the timing_resolution to less then 5 minutes is not suggested because it
+      is a limit for TaskJuggler.
+
+    :type timing_resolution: datetime.timedelta
+
     """
     __auto_name__ = False
     __tablename__ = 'Studios'
@@ -92,39 +118,161 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         primary_key=True,
     )
 
-    daily_working_hours = Column(Integer, default=8)
+    _timing_resolution = Column("timing_resolution", Interval)
+
+    is_scheduling = Column(Boolean, default=False)
+    is_scheduling_by_id = Column(
+        Integer,
+        ForeignKey('Users.id'),
+        doc='The id of the user who is scheduling the Studio projects right '
+            'now'
+    )
+    is_scheduling_by = relationship(
+        'User',
+        primaryjoin='Studios.c.last_scheduled_by_id==Users.c.id',
+        doc='The User who has last scheduled the Studio projects'
+    )
+    scheduling_started_at = Column(
+        DateTime,
+        doc='Stores when the current scheduling is started at, it is a good '
+            'measure for measuring if the last schedule is not correctly '
+            'finished'
+    )
+    last_scheduled_at = Column(
+        DateTime,
+        doc='Stores the last schedule date'
+    )
+    last_scheduled_by_id = Column(
+        Integer,
+        ForeignKey('Users.id'),
+        doc='The id of the user who has last scheduled the Studio projects'
+    )
+    last_scheduled_by = relationship(
+        'User',
+        primaryjoin='Studios.c.last_scheduled_by_id==Users.c.id',
+        doc='The User who has last scheduled the Studio projects'
+    )
+    last_schedule_message = Column(
+        PickleType,
+        doc='Holds the last schedule message, generally coming generated by '
+        'TaskJuggler'
+    )
 
     def __init__(self,
                  daily_working_hours=None,
                  now=None,
+                 timing_resolution=None,
                  **kwargs):
         super(Studio, self).__init__(**kwargs)
-        ScheduleMixin.__init__(self, **kwargs)
+        DateRangeMixin.__init__(self, **kwargs)
         WorkingHoursMixin.__init__(self, **kwargs)
-        # TODO: daily_working_hours should be in WorkingHours not in Studio
+        self.timing_resolution = timing_resolution
         self.daily_working_hours = daily_working_hours
         self._now = None
         self.now = self._validate_now(now)
         self._scheduler = None
 
-    # @reconstructor
-    # def __init_on_load__(self):
-    #     self._now = None
-    #     super(Studio, self).__init_on_load__()
+        # update defaults
+        self.update_defaults()
 
-    @validates('daily_working_hours')
-    def _validate_daily_working_hours(self, key, dwh):
-        """validates the given daily working hours value
+    @property
+    def daily_working_hours(self):
+        """a shortcut for Studio.working_hours.daily_working_hours
         """
-        if dwh is None:
-            dwh = defaults.daily_working_hours
+        return self.working_hours.daily_working_hours
+
+    @daily_working_hours.setter
+    def daily_working_hours(self, dwh):
+        """a shortcut for Studio.working_hours.daily_working_hours
+        """
+        self.working_hours.daily_working_hours = dwh
+
+    def update_defaults(self):
+        """updates the default values with the studio
+        """
+        # TODO: add update_defaults() to attribute edit/update methods,
+        #       so we will always have an up to date info about the working
+        #       hours.
+
+        logger.debug('updating defaults with Studio instance')
+        from stalker import defaults
+        try:
+            if self.daily_working_hours:
+                defaults.daily_working_hours = self.daily_working_hours
+                logger.debug(
+                    'updated defaults.daily_working_hours: %s' %
+                    defaults.daily_working_hours
+                )
+            else:
+                logger.debug('can not update defaults.daily_working_hours')
+        except AttributeError:
+            # The Studio and WorkingHours classes has changed from
+            # v0.2.3 to v0.2.5 and with this change it is simply
+            # no possible to initialize the db if we insist to update the
+            # defaults for daily_working_hours, because there is no
+            # WorkingHours._daily_working_hours in versions before than
+            # v0.2.5, so just skip it for at least the studio instance
+            # in the database has been updated.
+            logger.debug(
+                'Can not update defaults.daily_working_hours, WorkingHours '
+                'version mismatch'
+            )
+
+        if self.weekly_working_days:
+            defaults.weekly_working_days = self.weekly_working_days
+            logger.debug(
+                'updated defaults.weekly_working_days: %s' %
+                defaults.weekly_working_days
+            )
         else:
-            if not isinstance(dwh, int):
-                raise TypeError('%s.daily_working_hours should be an integer, '
-                                'not %s' %
-                                (self.__class__.__name__,
-                                 dwh.__class__.__name__))
-        return dwh
+            logger.debug('can not update defaults.weekly_working_days')
+
+        if self.weekly_working_hours:
+            defaults.weekly_working_hours = self.weekly_working_hours
+            logger.debug(
+                'updated defaults.weekly_working_hours: %s' %
+                defaults.weekly_working_hours
+            )
+        else:
+            logger.debug('can not update defaults.weekly_working_hours')
+
+        if self.yearly_working_days:
+            defaults.yearly_working_days = self.yearly_working_days
+            logger.debug(
+                'updated defaults.yearly_working_days: %s' %
+                defaults.yearly_working_days
+            )
+        else:
+            logger.debug('can not update defaults.yearly_working_days')
+
+        if self.timing_resolution:
+            defaults.timing_resolution = self.timing_resolution
+            logger.debug(
+                'updated defaults.timing_resolution: %s' %
+                defaults.timing_resolution
+            )
+        else:
+            logger.debug('can not update defaults.timing_resolution')
+
+        logger.debug("""done updating defaults:
+        daily_working_hours  : %(daily_working_hours)s
+        weekly_working_days  : %(weekly_working_days)s
+        weekly_working_hours : %(weekly_working_hours)s
+        yearly_working_days  : %(yearly_working_days)s
+        timing_resolution    : %(timing_resolution)s
+        """ % {
+            'daily_working_hours': defaults.daily_working_hours,
+            'weekly_working_days': defaults.weekly_working_days,
+            'weekly_working_hours': defaults.weekly_working_hours,
+            'yearly_working_days': defaults.yearly_working_days,
+            'timing_resolution': defaults.timing_resolution,
+        })
+
+    @reconstructor
+    def __init_on_load__(self):
+        """update defaults on load
+        """
+        self.update_defaults()
 
     def _validate_now(self, now_in):
         """validates the given now_in value
@@ -133,10 +281,11 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
             now_in = datetime.datetime.now()
 
         if not isinstance(now_in, datetime.datetime):
-            raise TypeError('%s.now attribute should be an instance of '
-                            'datetime.datetime, not %s' %
-                            (self.__class__.__name__,
-                             now_in.__class__.__name__))
+            raise TypeError(
+                '%s.now attribute should be an instance of datetime.datetime, '
+                'not %s' %
+                (self.__class__.__name__, now_in.__class__.__name__)
+            )
 
         return self.round_time(now_in)
 
@@ -162,10 +311,11 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """
         if scheduler_in is not None:
             if not isinstance(scheduler_in, SchedulerBase):
-                raise TypeError('%s.scheduler should be an instance of '
-                                'stalker.models.scheduler.SchedulerBase, not '
-                                '%s' % (self.__class__.__name__,
-                                        scheduler_in.__class__.__name__))
+                raise TypeError(
+                    '%s.scheduler should be an instance of '
+                    'stalker.models.scheduler.SchedulerBase, not %s' %
+                    (self.__class__.__name__, scheduler_in.__class__.__name__)
+                )
         return scheduler_in
 
     @property
@@ -186,7 +336,11 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """
         from jinja2 import Template
 
-        temp = Template(defaults.tjp_studio_template)
+        temp = Template(
+            defaults.tjp_studio_template,
+            trim_blocks=True,
+            lstrip_blocks=True
+        )
         start = time.time()
         rendered_template = temp.render({
             'studio': self,
@@ -202,7 +356,6 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """returns all the projects in the studio
         """
         from stalker import Project
-
         return Project.query.all()
 
     @property
@@ -210,23 +363,20 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """returns all the active projects in the studio
         """
         from stalker import Project
-
-        return Project.query.filter(Project.active == True).all()
+        return Project.query.filter(Project.active==True).all()
 
     @property
     def inactive_projects(self):
         """return all the inactive projects in the studio
         """
         from stalker import Project
-
-        return Project.query.filter(Project.active == False).all()
+        return Project.query.filter(Project.active==False).all()
 
     @property
     def departments(self):
         """returns all the departments in the studio
         """
         from stalker import Department
-
         return Department.query.all()
 
     @property
@@ -234,7 +384,6 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """returns all the users in the studio
         """
         from stalker import User
-
         return User.query.all()
 
     @property
@@ -243,25 +392,69 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """
         return Vacation.query.filter(Vacation.user==None).all()
 
-    def schedule(self):
+    def schedule(self, scheduled_by=None):
         """Schedules all the active projects in the studio. Needs a Scheduler,
         so before calling it set a scheduler by using the :attr:`.scheduler`
         attribute.
+
+        :param scheduled_by: A User instance who is doing the scheduling.
         """
         # check the scheduler first
         if self.scheduler is None or \
                 not isinstance(self.scheduler, SchedulerBase):
-            raise RuntimeError('There is no scheduler for this %s, please '
-                               'assign a scheduler to the %s.scheduler '
-                               'attribute, before calling %s.schedule()' %
-                               (self.__class__.__name__,
-                                self.__class__.__name__,
-                                self.__class__.__name__))
+            raise RuntimeError(
+                'There is no scheduler for this %(class)s, please assign a '
+                'scheduler to the %(class)s.scheduler attribute, before '
+                'calling %(class)s.schedule()' %
+                {
+                    'class': self.__class__.__name__
+                }
+            )
+
+        # check if the studio is already scheduling
+        if self.is_scheduling:
+            raise RuntimeError(
+                'The Studio is being scheduling right now by %(user)s, please '
+                'wait until the current scheduling is done!' % {
+                    'user': self.is_scheduling_by.name
+                }
+            )
+        else:
+            # set to self.is_scheduling
+            self.is_scheduling = True
+            self.is_scheudling_by = scheduled_by
+
+        self.scheduling_started_at = datetime.datetime.now()
 
         # run the scheduler
         self.scheduler.studio = self
         start = time.time()
-        result = self.scheduler.schedule()
+
+        # commit before scheduling
+        DBSession.commit()
+
+        result = None
+        try:
+            result = self.scheduler.schedule()
+        finally:
+            # in any case set is_scheduling to False
+            self.is_scheduling = False
+            self.is_scheduling_by = None
+
+            # also store the result
+            # if result:
+            self.last_schedule_message = result
+
+            # And the date the schedule is completed
+            # TODO: convert to UTC time
+            self.last_scheduled_at = datetime.datetime.now()
+
+            # and who has done the scheduling
+            if scheduled_by:
+                self.last_scheduled_by = scheduled_by
+
+            DBSession.commit()
+
         end = time.time()
         logger.debug('scheduling took %s seconds' % (end - start))
         return result
@@ -284,6 +477,61 @@ class Studio(Entity, ScheduleMixin, WorkingHoursMixin):
         """
         return self.working_hours.yearly_working_days
 
+    def to_unit(self, from_timing, from_unit, to_unit, working_hours=True):
+        """converts the given timing and unit to the desired unit
+        if working_hours=True then the given timing is considered as working
+        hours
+        """
+        raise NotImplementedError('this is not implemented yet')
+
+    def _timing_resolution_getter(self):
+        """returns the timing_resolution
+        """
+        return self._timing_resolution
+
+    def _timing_resolution_setter(self, res_in):
+        """sets the timing_resolution
+        """
+        self._timing_resolution = self._validate_timing_resolution(res_in)
+        logger.debug('self._timing_resolution: %s' % self._timing_resolution)
+        # update date values
+        if self.start and self.end and self.duration:
+            self._start, self._end, self._duration = \
+                self._validate_dates(
+                    self.round_time(self.start),
+                    self.round_time(self.end),
+                    None
+                )
+
+    timing_resolution = synonym(
+        '_timing_resolution',
+        descriptor=property(
+            _timing_resolution_getter,
+            _timing_resolution_setter,
+            doc="""The timing_resolution of this object.
+
+            Can be set to any value that is representable with
+            datetime.timedelta. The default value is 1 hour. Whenever it is
+            changed the start, end and duration values will be updated.
+            """
+        )
+    )
+
+    def _validate_timing_resolution(self, timing_resolution):
+        """validates the given timing_resolution value
+        """
+        if timing_resolution is None:
+            timing_resolution = defaults.timing_resolution
+
+        if not isinstance(timing_resolution, datetime.timedelta):
+            raise TypeError(
+                '%s.timing_resolution should be an instance of '
+                'datetime.timedelta not, %s' %
+                (self.__class__.__name__, timing_resolution.__class__.__name__)
+            )
+
+        return timing_resolution
+
 
 class WorkingHours(object):
     """A helper class to manage Studio working hours.
@@ -341,17 +589,22 @@ class WorkingHours(object):
       value is going to be used.
     """
 
-    def __init__(self, working_hours=None, **kwargs):
+    def __init__(self,
+                 working_hours=None,
+                 daily_working_hours=None,
+                 **kwargs):
         if working_hours is None:
             working_hours = defaults.working_hours
         self._wh = None
         self.working_hours = self._validate_working_hours(working_hours)
+        self._daily_working_hours = None
+        self.daily_working_hours = daily_working_hours
 
     def __eq__(self, other):
         """equality test
         """
         return isinstance(other, WorkingHours) and \
-               other.working_hours == self.working_hours
+            other.working_hours == self.working_hours
 
     def __getitem__(self, item):
         if isinstance(item, int):
@@ -366,27 +619,30 @@ class WorkingHours(object):
         elif isinstance(key, str):
             # check if key is in
             if key not in defaults.day_order:
-                raise KeyError('%s accepts only %s as key, not %s' %
-                               (self.__class__.__name__, defaults.day_order,
-                                key))
+                raise KeyError(
+                    '%s accepts only %s as key, not %s' %
+                    (self.__class__.__name__, defaults.day_order, key)
+                )
             self._wh[key] = value
 
     def _validate_working_hours(self, wh_in):
         """validates the given working hours
         """
         if not isinstance(wh_in, dict):
-            raise TypeError('%s.working_hours should be a dictionary, not %s' %
-                            (self.__class__.__name__,
-                             wh_in.__class__.__name__))
+            raise TypeError(
+                '%s.working_hours should be a dictionary, not %s' %
+                (self.__class__.__name__, wh_in.__class__.__name__)
+            )
 
         for key in wh_in.keys():
             if not isinstance(wh_in[key], list):
-                raise TypeError('%s.working_hours should be a dictionary with '
-                                'keys "mon, tue, wed, thu, fri, sat, sun" '
-                                'and the values should a list of lists of '
-                                'two integers like [[540, 720], [800, 1080]], '
-                                'not %s' % (self.__class__.__name__,
-                                            wh_in[key].__class__.__name__))
+                raise TypeError(
+                    '%s.working_hours should be a dictionary with keys "mon, '
+                    'tue, wed, thu, fri, sat, sun" and the values should a '
+                    'list of lists of two integers like [[540, 720], [800, '
+                    '1080]], not %s' %
+                    (self.__class__.__name__, wh_in[key].__class__.__name__)
+                )
 
             # validate item values
             self._validate_wh_value(wh_in[key])
@@ -501,19 +757,50 @@ class WorkingHours(object):
     def yearly_working_days(self):
         """returns the total working days in a year
         """
-        return self.weekly_working_days * 52.1428
+        return int(ceil(self.weekly_working_days * 52.1428))
+
+    def _validate_daily_working_hours(self, dwh):
+        """validates the given daily working hours value
+        """
+        if dwh is None:
+            dwh = defaults.daily_working_hours
+
+        if not isinstance(dwh, int):
+            raise TypeError(
+                '%s.daily_working_hours should be an integer, not %s' %
+                (self.__class__.__name__, dwh.__class__.__name__)
+            )
+
+        if dwh <= 0 or dwh > 24:
+            raise ValueError(
+                '%s.daily_working_hours should be a positive integer value '
+                'greater than 0 and smaller than or equal to 24'
+            )
+        return dwh
+
+    @property
+    def daily_working_hours(self):
+        """getter for daily_working_hours attribute
+        """
+        return self._daily_working_hours
+
+    @daily_working_hours.setter
+    def daily_working_hours(self, dwh):
+        """setter for daily_working_hours attribute
+        """
+        self._daily_working_hours = self._validate_daily_working_hours(dwh)
 
 
-class Vacation(SimpleEntity, ScheduleMixin):
+class Vacation(SimpleEntity, DateRangeMixin):
     """Vacation is the way to manage the User vacations.
 
     :param user: The user of this vacation. Should be an instance of
-      :class:`~stalker.models.auth.User` if skipped or given as None the
+      :class:`.User` if skipped or given as None the
       Vacation is considered as a Studio vacation and applies to all Users.
 
     :param start: The start datetime of the vacation. Is is an
       datetime.datetime instance. When skipped it will be set to the rounded
-      value of 
+      value of.
 
     :param end: The end datetime of the vacation. It is an datetime.datetime
       instance.
@@ -536,7 +823,7 @@ class Vacation(SimpleEntity, ScheduleMixin):
         back_populates='vacations',
         doc="""The User of this Vacation.
 
-        Accepts:class:`~stalker.models.auth.User` instance.
+        Accepts :class:`.User` instance.
         """
     )
 
@@ -544,7 +831,7 @@ class Vacation(SimpleEntity, ScheduleMixin):
         kwargs['start'] = start
         kwargs['end'] = end
         super(Vacation, self).__init__(**kwargs)
-        ScheduleMixin.__init__(self, **kwargs)
+        DateRangeMixin.__init__(self, **kwargs)
         self.user = user
 
     @validates('user')
@@ -554,9 +841,11 @@ class Vacation(SimpleEntity, ScheduleMixin):
         if user is not None:
             from stalker import User
             if not isinstance(user, User):
-                raise TypeError('%s.user should be an instance of '
-                                'stalker.models.auth.User, not %s' %
-                                (self.__class__.__name__, user.__class__.__name__))
+                raise TypeError(
+                    '%s.user should be an instance of '
+                    'stalker.models.auth.User, not %s' %
+                    (self.__class__.__name__, user.__class__.__name__)
+                )
         return user
 
     @property
