@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -21,17 +21,12 @@
 import datetime
 import unittest2
 
-from stalker import (Asset, Entity, ImageFormat, Link, Project, Repository,
-                     Sequence, Shot, Status, StatusList, Structure, Task, Type,
-                     User, db, Ticket)
-from stalker import config
-
-defaults = config.Config()
-
-from stalker.db.session import DBSession
-
-import logging
 from stalker import log
+from stalker.db.session import DBSession
+from stalker import (db, Asset, Entity, ImageFormat, Link, Project, Repository,
+                     Sequence, Shot, Status, StatusList, Structure, Task, Type,
+                     User, Ticket)
+import logging
 
 logger = logging.getLogger('stalker.models.project')
 logger.setLevel(log.logging_level)
@@ -46,13 +41,12 @@ class ProjectTestCase(unittest2.TestCase):
         """set up the test for class
         """
         DBSession.remove()
-        DBSession.configure(extension=None)
 
     @classmethod
     def tearDownClass(cls):
         """clean up the test
         """
-        DBSession.configure(extension=None)
+        DBSession.remove()
 
     def tearDown(self):
         """tearDown the tests
@@ -62,14 +56,7 @@ class ProjectTestCase(unittest2.TestCase):
     def setUp(self):
         """setup the test
         """
-        DBSession.remove()
-        DBSession.configure(extension=None)
-        self.TEST_DATABASE_URI = "sqlite:///:memory:"
-
-        db.setup({
-            "sqlalchemy.url": self.TEST_DATABASE_URI,
-            "sqlalchemy.echo": False,
-        })
+        db.setup()
         #db.init()  # for tickets
 
         # create test objects
@@ -154,26 +141,29 @@ class ProjectTestCase(unittest2.TestCase):
         )
 
         # statuses
-        self.test_status1 = Status(name="Status1", code="S1")
-        self.test_status2 = Status(name="Status2", code="S2")
-        self.test_status3 = Status(name="Status3", code="S3")
-        self.test_status4 = Status(name="Status4", code="S4")
-        self.test_status5 = Status(name="Status5", code="S5")
+        self.status_new = Status(name="New", code="NEW")
+        self.status_wfd = Status(name="Waiting For Dependency", code="WFD")
+        self.status_rts = Status(name="Ready To Start", code="RTS")
+        self.status_wip = Status(name="Work In Progress", code="WIP")
+        self.status_prev = Status(name="Pending Review", code="PREV")
+        self.status_hrev = Status(name="Has Revision", code="HREV")
+        self.status_drev = Status(name="Dependency Has Revision", code="DREV")
+        self.status_oh = Status(name="On Hold", code="OH")
+        self.status_stop = Status(name="Stopped", code="STOP")
+        self.status_cmpl = Status(name="Completed", code="CMPL")
 
         # status list for project
         self.project_status_list = StatusList(
             name="Project Statuses",
             target_entity_type=Project,
             statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4,
-                self.test_status5,
+                self.status_new,
+                self.status_wip,
+                self.status_cmpl,
             ],
         )
 
-        self.test_imageFormat = ImageFormat(
+        self.test_image_format = ImageFormat(
             name="HD",
             width=1920,
             height=1080,
@@ -225,7 +215,7 @@ class ProjectTestCase(unittest2.TestCase):
             'code': 'tp',
             "description": "This is a project object for testing purposes",
             "lead": self.test_lead,
-            "image_format": self.test_imageFormat,
+            "image_format": self.test_image_format,
             "fps": 25,
             "type": self.test_project_type,
             "structure": self.test_project_structure,
@@ -243,11 +233,15 @@ class ProjectTestCase(unittest2.TestCase):
         self.sequence_status_list = StatusList(
             name="Sequence Statuses",
             statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4,
-                self.test_status5,
+                self.status_rts,
+                self.status_wfd,
+                self.status_wip,
+                self.status_prev,
+                self.status_hrev,
+                self.status_drev,
+                self.status_oh,
+                self.status_stop,
+                self.status_cmpl,
             ],
             target_entity_type=Sequence
         )
@@ -311,11 +305,15 @@ class ProjectTestCase(unittest2.TestCase):
         self.shot_status_list = StatusList(
             name="Shot Status List",
             statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4,
-                self.test_status5,
+                self.status_rts,
+                self.status_wfd,
+                self.status_wip,
+                self.status_prev,
+                self.status_hrev,
+                self.status_drev,
+                self.status_oh,
+                self.status_stop,
+                self.status_cmpl,
             ],
             target_entity_type=Shot,
         )
@@ -353,11 +351,15 @@ class ProjectTestCase(unittest2.TestCase):
         self.asset_status_list = StatusList(
             name="Asset Status List",
             statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4,
-                self.test_status5,
+                self.status_rts,
+                self.status_wfd,
+                self.status_wip,
+                self.status_prev,
+                self.status_hrev,
+                self.status_drev,
+                self.status_oh,
+                self.status_stop,
+                self.status_cmpl,
             ],
             target_entity_type=Asset,
         )
@@ -416,11 +418,15 @@ class ProjectTestCase(unittest2.TestCase):
         self.task_status_list = StatusList(
             name="Task Status List",
             statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4,
-                self.test_status5,
+                self.status_rts,
+                self.status_wfd,
+                self.status_wip,
+                self.status_prev,
+                self.status_hrev,
+                self.status_drev,
+                self.status_oh,
+                self.status_stop,
+                self.status_cmpl,
             ],
             target_entity_type=Task,
         )
@@ -782,7 +788,7 @@ class ProjectTestCase(unittest2.TestCase):
             self.assertRaises(TypeError, Project, **self.kwargs)
 
         # and a proper image format
-        self.kwargs["image_format"] = self.test_imageFormat
+        self.kwargs["image_format"] = self.test_image_format
         new_project = Project(**self.kwargs)
         self.assertIsInstance(new_project, Project)
 
@@ -802,7 +808,7 @@ class ProjectTestCase(unittest2.TestCase):
             )
 
         # and a proper image format
-        self.test_project.image_format = self.test_imageFormat
+        self.test_project.image_format = self.test_image_format
 
     def test_image_format_attribute_works_properly(self):
         """testing if the image_format attribute is working properly
@@ -1106,18 +1112,20 @@ class ProjectTestCase(unittest2.TestCase):
         """testing if the StatusMixin part is initialized correctly
         """
         status1 = Status(name="On Hold", code="OH")
-        status2 = Status(name="Complete", code="CMPLT")
+        status2 = Status(name="Complete", code="CMPL")
 
-        status_list = StatusList(name="Project Statuses",
-                                 statuses=[status1, status2],
-                                 target_entity_type=Project)
+        status_list = StatusList(
+            name="Project Statuses",
+            statuses=[status1, status2],
+            target_entity_type=Project
+        )
         self.kwargs["status"] = 0
         self.kwargs["status_list"] = status_list
         new_project = Project(**self.kwargs)
         self.assertEqual(new_project.status_list, status_list)
 
     def test_ScheduleMixin_initialization(self):
-        """testing if the ScheduleMixin part is initialized correctly
+        """testing if the DateRangeMixin part is initialized correctly
         """
         start = \
             datetime.datetime(2013, 3, 22, 4, 0) + datetime.timedelta(days=25)
@@ -1296,302 +1304,319 @@ class ProjectTestCase(unittest2.TestCase):
     def test_to_tjp_is_working_properly(self):
         """testing if the to_tjp attribute is working properly
         """
-        self.maxDiff = None
         from jinja2 import Template
 
-        expected_tjp_temp = Template("""task Project_28 "Test Project" {
-            
-                task Sequence_29 "Seq1" {
+        expected_tjp_temp = Template("""
+task Project_33 "Test Project" {
         
+task Sequence_34 "Seq1" {
+
+    
+            
             
             effort 1.0h
-            allocate User_1
-        }
-        
+            allocate User_1             
+}        
+task Sequence_35 "Seq2" {
+
+    
             
-                task Sequence_30 "Seq2" {
-        
             
             effort 1.0h
-            allocate User_8
-        }
-        
+            allocate User_12             
+}        
+task Sequence_36 "Seq3" {
+
+    
             
-                task Sequence_31 "Seq3" {
-        
             
             effort 1.0h
-            allocate User_9
-        }
-        
+            allocate User_13             
+}        
+task Sequence_37 "Seq4" {
+
+    
+    
+task Task_53 "Test Task 4" {
+
+    
             
-                task Sequence_32 "Seq4" {
-        
-                task Task_48 "Test Task 4" {
-        
             
             effort 1.0h
-            allocate User_15
-        }
-        
-                task Task_49 "Test Task 5" {
-        
+            allocate User_19             
+}
+task Task_54 "Test Task 5" {
+
+    
+            
             
             effort 1.0h
-            allocate User_16
-        }
-        
-                task Task_50 "Test Task 6" {
-        
+            allocate User_20             
+}
+task Task_55 "Test Task 6" {
+
+    
+            
             
             effort 1.0h
-            allocate User_17
-        }
-        
-        }
-        
+            allocate User_21             
+}
+}        
+task Sequence_38 "Seq5" {
+
+    
+    
+task Task_56 "Test Task 7" {
+
+    
             
-                task Sequence_33 "Seq5" {
-        
-                task Task_51 "Test Task 7" {
-        
             
             effort 1.0h
-            allocate User_18
-        }
-        
-                task Task_52 "Test Task 8" {
-        
+            allocate User_22             
+}
+task Task_57 "Test Task 8" {
+
+    
+            
             
             effort 1.0h
-            allocate User_19
-        }
-        
-                task Task_53 "Test Task 9" {
-        
+            allocate User_23             
+}
+task Task_58 "Test Task 9" {
+
+    
+            
             
             effort 1.0h
-            allocate User_20
-        }
-        
-        }
-        
+            allocate User_24             
+}
+}        
+task Sequence_39 "Seq6" {
+
+    
+                        
+}        
+task Sequence_40 "Seq7" {
+
+    
+                        
+}        
+task Shot_41 "{{shot1.name}}" {
+
+    
+    
+task Task_59 "Test Task 10" {
+
+    
             
-                task Sequence_34 "Seq6" {
-        
-            
-        }
-        
-            
-                task Sequence_35 "Seq7" {
-        
-            
-        }
-        
-            
-                task Shot_36 "{{shot1.name}}" {
-        
-                task Task_54 "Test Task 10" {
-        
             
             effort 10.0h
-            allocate User_21
-        }
-        
-                task Task_55 "Test Task 11" {
-        
+            allocate User_25             
+}
+task Task_60 "Test Task 11" {
+
+    
+            
             
             effort 1.0h
-            allocate User_1, User_8
-        }
-        
-                task Task_56 "Test Task 12" {
-        
+            allocate User_1 , User_12             
+}
+task Task_61 "Test Task 12" {
+
+    
+            
             
             effort 1.0h
-            allocate User_9, User_15
-        }
-        
-        }
-        
+            allocate User_13 , User_19             
+}
+}        
+task Shot_42 "{{shot2.name}}" {
+
+    
+    
+task Task_62 "Test Task 13" {
+
+    
             
-                task Shot_37 "{{shot2.name}}" {
-        
-                task Task_57 "Test Task 13" {
-        
             
             effort 1.0h
-            allocate User_16, User_17
-        }
-        
-                task Task_58 "Test Task 14" {
-        
+            allocate User_20 , User_21             
+}
+task Task_63 "Test Task 14" {
+
+    
+            
             
             effort 1.0h
-            allocate User_18, User_19
-        }
-        
-                task Task_59 "Test Task 15" {
-        
+            allocate User_22 , User_23             
+}
+task Task_64 "Test Task 15" {
+
+    
+            
             
             effort 1.0h
-            allocate User_20, User_21
-        }
-        
-        }
-        
+            allocate User_24 , User_25             
+}
+}        
+task Shot_43 "{{shot3.name}}" {
+
+    
+    
+task Task_65 "Test Task 16" {
+
+    
             
-                task Shot_38 "{{shot3.name}}" {
-        
-                task Task_60 "Test Task 16" {
-        
             
             effort 1.0h
-            allocate User_1, User_8, User_9
-        }
-        
-                task Task_61 "Test Task 17" {
-        
+            allocate User_1 , User_12 , User_13             
+}
+task Task_66 "Test Task 17" {
+
+    
+            
             
             effort 1.0h
-            allocate User_15, User_16, User_17
-        }
-        
-                task Task_62 "Test Task 18" {
-        
+            allocate User_19 , User_20 , User_21             
+}
+task Task_67 "Test Task 18" {
+
+    
+            
             
             effort 1.0h
-            allocate User_18, User_19, User_20
-        }
-        
-        }
-        
+            allocate User_22 , User_23 , User_24             
+}
+}        
+task Shot_44 "{{shot4.name}}" {
+
+    
+    
+task Task_68 "Test Task 19" {
+
+    
             
-                task Shot_39 "{{shot4.name}}" {
-        
-                task Task_63 "Test Task 19" {
-        
             
             effort 1.0h
-            allocate User_1, User_8, User_21
-        }
-        
-                task Task_64 "Test Task 20" {
-        
+            allocate User_1 , User_12 , User_25             
+}
+task Task_69 "Test Task 20" {
+
+    
+            
             
             effort 1.0h
-            allocate User_9, User_15, User_16
-        }
-        
-                task Task_65 "Test Task 21" {
-        
+            allocate User_13 , User_19 , User_20             
+}
+task Task_70 "Test Task 21" {
+
+    
+            
             
             effort 1.0h
-            allocate User_17, User_18, User_19
-        }
-        
-        }
-        
+            allocate User_21 , User_22 , User_23             
+}
+}        
+task Asset_45 "Test Asset 1" {
+
+    
             
-                task Asset_40 "Test Asset 1" {
-        
             
             effort 1.0h
-            allocate User_8
-        }
-        
+            allocate User_12             
+}        
+task Asset_46 "Test Asset 2" {
+
+    
+                        
+}        
+task Asset_47 "Test Asset 3" {
+
+    
+                        
+}        
+task Asset_48 "Test Asset 4" {
+
+    
+    
+task Task_71 "Test Task 22" {
+
+    
             
-                task Asset_41 "Test Asset 2" {
-        
-            
-        }
-        
-            
-                task Asset_42 "Test Asset 3" {
-        
-            
-        }
-        
-            
-                task Asset_43 "Test Asset 4" {
-        
-                task Task_66 "Test Task 22" {
-        
             
             effort 1.0h
-            allocate User_1, User_20, User_21
-        }
-        
-                task Task_67 "Test Task 23" {
-        
+            allocate User_1 , User_24 , User_25             
+}
+task Task_72 "Test Task 23" {
+
+    
+            
             
             effort 1.0h
-            allocate User_8, User_9
-        }
-        
-                task Task_68 "Test Task 24" {
-        
+            allocate User_12 , User_13             
+}
+task Task_73 "Test Task 24" {
+
+    
+            
             
             effort 1.0h
-            allocate User_15, User_16
-        }
-        
-        }
-        
+            allocate User_19 , User_20             
+}
+}        
+task Asset_49 "Test Asset 5" {
+
+    
+    
+task Task_74 "Test Task 25" {
+
+    
             
-                task Asset_44 "Test Asset 5" {
-        
-                task Task_69 "Test Task 25" {
-        
             
             effort 1.0h
-            allocate User_17, User_18
-        }
-        
-                task Task_70 "Test Task 26" {
-        
+            allocate User_21 , User_22             
+}
+task Task_75 "Test Task 26" {
+
+    
+            
             
             effort 1.0h
-            allocate User_19, User_20
-        }
-        
-                task Task_71 "Test Task 27" {
-        
+            allocate User_23 , User_24             
+}
+task Task_76 "Test Task 27" {
+
+    
+            
             
             effort 1.0h
-            allocate User_1, User_21
-        }
-        
-        }
-        
+            allocate User_1 , User_25             
+}
+}        
+task Task_50 "Test Task 1" {
+
+    
             
-                task Task_45 "Test Task 1" {
-        
             
             effort 1.0h
-            allocate User_1
-        }
-        
+            allocate User_1             
+}        
+task Task_51 "Test Task 2" {
+
+    
             
-                task Task_46 "Test Task 2" {
-        
             
             effort 1.0h
-            allocate User_8
-        }
-        
+            allocate User_12             
+}        
+task Task_52 "Test Task 3" {
+
+    
             
-                task Task_47 "Test Task 3" {
-        
             
             effort 1.0h
-            allocate User_9
-        }
-        
-            
-        }
-        
-""")
+            allocate User_13             
+}}""")
         expected_tjp = expected_tjp_temp.render({
             'shot1': self.test_shot1,
             'shot2': self.test_shot2,
@@ -1599,8 +1624,8 @@ class ProjectTestCase(unittest2.TestCase):
             'shot4': self.test_shot4
         })
 
-        print expected_tjp
-        print "-----------------"
+        # print expected_tjp
+        # print "-----------------"
         print self.test_project.to_tjp
 
         self.assertEqual(self.test_project.to_tjp, expected_tjp)
@@ -1645,7 +1670,7 @@ class ProjectTestCase(unittest2.TestCase):
         # create some time logs
         from stalker import TimeLog
 
-        tlog1 = TimeLog(
+        TimeLog(
             task=self.test_task1,
             resource=self.test_task1.resources[0],
             start=datetime.datetime(2013, 8, 1, 1, 0),
@@ -1654,7 +1679,7 @@ class ProjectTestCase(unittest2.TestCase):
         self.assertEqual(self.test_project.total_logged_seconds, 3600)
 
         # add more time logs
-        tlog2 = TimeLog(
+        TimeLog(
             task=self.test_seq1,
             resource=self.test_seq1.resources[0],
             start=datetime.datetime(2013, 8, 1, 2, 0),
@@ -1663,7 +1688,7 @@ class ProjectTestCase(unittest2.TestCase):
         self.assertEqual(self.test_project.total_logged_seconds, 7200)
 
         # create more deeper time logs
-        tlog3 = TimeLog(
+        TimeLog(
             task=self.test_task10,
             resource=self.test_task10.resources[0],
             start=datetime.datetime(2013, 8, 1, 3, 0),
@@ -1672,7 +1697,7 @@ class ProjectTestCase(unittest2.TestCase):
         self.assertEqual(self.test_project.total_logged_seconds, 18000)
 
         # create a time log for one asset
-        tlog4 = TimeLog(
+        TimeLog(
             task=self.test_asset1,
             resource=self.test_asset1.resources[0],
             start=datetime.datetime(2013, 8, 1, 6, 0),
@@ -1749,7 +1774,6 @@ class ProjectTestCase(unittest2.TestCase):
 
         self.assertEqual(self.test_project.schedule_seconds, 44 * 3600)
 
-
     def test_percent_complete_attribute_is_read_only(self):
         """testing if the percent_complete is a read-only attribute
         """
@@ -1778,7 +1802,7 @@ class ProjectTestCase(unittest2.TestCase):
         # create some time logs
         from stalker import TimeLog
 
-        tlog1 = TimeLog(
+        TimeLog(
             task=self.test_task1,
             resource=self.test_task1.resources[0],
             start=datetime.datetime(2013, 8, 1, 1, 0),
@@ -1798,13 +1822,12 @@ class ProjectTicketsTestCase(unittest2.TestCase):
         """set up the test for class
         """
         DBSession.remove()
-        DBSession.configure(extension=None)
 
     @classmethod
     def tearDownClass(cls):
         """clean up the test
         """
-        DBSession.configure(extension=None)
+        DBSession.remove()
 
     def tearDown(self):
         """tearDown the tests
@@ -1814,14 +1837,7 @@ class ProjectTicketsTestCase(unittest2.TestCase):
     def setUp(self):
         """setup the test
         """
-        DBSession.remove()
-        DBSession.configure(extension=None)
-        self.TEST_DATABASE_URI = "sqlite:///:memory:"
-
-        db.setup({
-            "sqlalchemy.url": self.TEST_DATABASE_URI,
-            "sqlalchemy.echo": False,
-        })
+        db.setup()
         db.init()  # for tickets
 
         # create test objects
@@ -1925,7 +1941,7 @@ class ProjectTicketsTestCase(unittest2.TestCase):
             ],
         )
 
-        self.test_imageFormat = ImageFormat(
+        self.test_image_format = ImageFormat(
             name="HD",
             width=1920,
             height=1080,
@@ -1977,7 +1993,7 @@ class ProjectTicketsTestCase(unittest2.TestCase):
             'code': 'tp',
             "description": "This is a project object for testing purposes",
             "lead": self.test_lead,
-            "image_format": self.test_imageFormat,
+            "image_format": self.test_image_format,
             "fps": 25,
             "type": self.test_project_type,
             "structure": self.test_project_structure,
@@ -2150,13 +2166,12 @@ class ProjectTicketsTestCase(unittest2.TestCase):
                 self.test_ticket5,
                 self.test_ticket6,
                 self.test_ticket7,
-                self.test_ticket8,
+                self.test_ticket8
             ]
         )
 
         # close a couple of them
-        from stalker.models.ticket import (FIXED, CANTFIX, WONTFIX, DUPLICATE,
-                                           WORKSFORME, INVALID)
+        from stalker.models.ticket import (FIXED, CANTFIX, INVALID)
 
         self.test_ticket1.resolve(self.test_user1, FIXED)
         self.test_ticket2.resolve(self.test_user1, INVALID)
